@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 """
 Model configuration in application ``django_mailbox`` for administration
@@ -10,6 +9,7 @@ import logging
 
 from django.conf import settings
 from django.contrib import admin
+from django.utils.translation import gettext_lazy as _
 
 from django_mailbox.models import MessageAttachment, Message, Mailbox
 from django_mailbox.signals import message_received
@@ -20,10 +20,10 @@ logger = logging.getLogger(__name__)
 
 
 def get_new_mail(mailbox_admin, request, queryset):
-    for mailbox in queryset.all():
-        logger.debug('Receiving mail for %s' % mailbox)
-        mailbox.get_new_mail()
-get_new_mail.short_description = 'Get new mail'
+    queryset.get_new_mail()
+
+
+get_new_mail.short_description = _('Get new mail')
 
 
 def resend_message_received_signal(message_admin, request, queryset):
@@ -33,7 +33,7 @@ def resend_message_received_signal(message_admin, request, queryset):
 
 
 resend_message_received_signal.short_description = (
-    'Re-send message received signal'
+    _('Re-send message received signal')
 )
 
 
@@ -43,7 +43,9 @@ class MailboxAdmin(admin.ModelAdmin):
         'uri',
         'from_email',
         'active',
+        'last_polling',
     )
+    readonly_fields = ['last_polling', ]
     actions = [get_new_mail]
 
 
@@ -61,13 +63,15 @@ class MessageAdmin(admin.ModelAdmin):
     def attachment_count(self, msg):
         return msg.attachments.count()
 
+    attachment_count.short_description = _('Attachment count')
+
     def subject(self, msg):
         return convert_header_to_unicode(msg.subject)
 
     def envelope_headers(self, msg):
         email = msg.get_email_object()
         return '\n'.join(
-            [('%s: %s' % (h, v)) for h, v in email.items()]
+            [('{}: {}'.format(h, v)) for h, v in email.items()]
         )
 
     inlines = [
@@ -101,6 +105,7 @@ class MessageAdmin(admin.ModelAdmin):
     )
     actions = [resend_message_received_signal]
 
+
 if getattr(settings, 'DJANGO_MAILBOX_ADMIN_ENABLED', True):
     admin.site.register(Message, MessageAdmin)
     admin.site.register(MessageAttachment, MessageAttachmentAdmin)
