@@ -1,7 +1,10 @@
 import email
 import logging
-import rfc822
 import sys
+try:
+    from email import utils
+except ImportError:
+    import rfc822 as utils
 
 from django.core.management.base import BaseCommand
 
@@ -14,7 +17,14 @@ logging.basicConfig(level=logging.INFO)
 
 class Command(BaseCommand):
     args = "<[Mailbox Name (optional)]>"
-    command = "Receive incoming mail via stdin"
+    help = "Receive incoming mail via stdin"
+
+    def add_arguments(self, parser):
+        parser.add_argument(
+            'mailbox_name',
+            nargs='?',
+            help="The name of the mailbox that will receive the message"
+        )
 
     def handle(self, mailbox_name=None, *args, **options):
         message = email.message_from_string(sys.stdin.read())
@@ -38,5 +48,5 @@ class Command(BaseCommand):
         return mailbox
 
     def get_mailbox_for_message(self, message):
-        email_address = rfc822.parseaddr(message['to'])[1][0:255]
+        email_address = utils.parseaddr(message['to'])[1][0:255]
         return self.get_mailbox_by_name(email_address)
