@@ -1,14 +1,15 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 """
 Models declaration for application ``django_mailbox``.
 """
-
+import gzip
 from email.encoders import encode_base64
 from email.message import Message as EmailMessage
 from email.utils import formatdate, parseaddr
+from urllib.parse import parse_qs, unquote, urlparse
 from quopri import encode as encode_quopri
+from io import BytesIO
 import base64
 import email
 import logging
@@ -16,78 +17,54 @@ import mimetypes
 import os.path
 import sys
 import uuid
+from tempfile import NamedTemporaryFile
 
-import six
-from six.moves.urllib.parse import parse_qs, unquote, urlparse
-
-from django.conf import settings
-from django.core.files.base import ContentFile
+import django
+from django.conf import settings as django_settings
+from django.core.files.base import ContentFile, File
 from django.core.mail.message import make_msgid
 from django.db import models
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext_lazy as _
+from django.utils.timezone import now
 
-from .utils import convert_header_to_unicode, get_body_from_message
+from django_mailbox import utils
 from django_mailbox.signals import message_received
 from django_mailbox.transports import Pop3Transport, ImapTransport, \
     MaildirTransport, MboxTransport, BabylTransport, MHTransport, \
     MMDFTransport, GmailImapTransport
 
-
 logger = logging.getLogger(__name__)
 
 
-STRIP_UNALLOWED_MIMETYPES = getattr(
-    settings,
-    'DJANGO_MAILBOX_STRIP_UNALLOWED_MIMETYPES',
-    False
-)
-
-ALLOWED_MIMETYPES = getattr(
-    settings,
-    'DJANGO_MAILBOX_ALLOWED_MIMETYPES',
-    [
-        'text/plain',
-        'text/html'
-    ]
-)
-
-TEXT_STORED_MIMETYPES = getattr(
-    settings,
-    'DJANGO_MAILBOX_TEXT_STORED_MIMETYPES',
-    [
-        'text/plain',
-        'text/html'
-    ]
-)
-
-ALTERED_MESSAGE_HEADER = getattr(
-    settings,
-    'DJANGO_MAILBOX_ALTERED_MESSAGE_HEADER',
-    'X-Django-Mailbox-Altered-Message'
-)
-
-ATTACHMENT_INTERPOLATION_HEADER = getattr(
-    settings,
-    'DJANGO_MAILBOX_ATTACHMENT_INTERPOLATION_HEADER',
-    'X-Django-Mailbox-Interpolate-Attachment'
-)
+class MailboxQuerySet(models.QuerySet):
+    def get_new_mail(self):
+        count = 0
+        for mailbox in self.all():
+            logger.debug('Receiving mail for %s' % mailbox)
+            count += sum(1 for i in mailbox.get_new_mail())
+        logger.debug('Received %d %s.', count, 'mails' if count != 1 else 'mail')
 
 
-class ActiveMailboxManager(models.Manager):
+class MailboxManager(models.Manager):
     def get_queryset(self):
-        return super(ActiveMailboxManager, self).get_queryset().filter(
+        return MailboxQuerySet(self.model, using=self._db)
+
+
+class ActiveMailboxManager(MailboxManager):
+    def get_queryset(self):
+        return super().get_queryset().filter(
             active=True,
         )
 
 
 class Mailbox(models.Model):
     name = models.CharField(
-        _(u'Name'),
+        _('Name'),
         max_length=255,
     )
 
     uri = models.CharField(
-        _(u'URI'),
+        _('URI'),
         max_length=255,
         help_text=(_(
             "Example: imap+ssl://myusername:mypassword@someserver <br />"
@@ -105,7 +82,7 @@ class Mailbox(models.Model):
     )
 
     from_email = models.CharField(
-        _(u'From email'),
+        _('From email'),
         max_length=255,
         help_text=(_(
             "Example: MailBot &lt;mailbot@yourdomain.com&gt;<br />"
@@ -122,7 +99,7 @@ class Mailbox(models.Model):
     )
 
     active = models.BooleanField(
-        _(u'Active'),
+        _('Active'),
         help_text=(_(
             "Check this e-mail inbox for new e-mail messages during polling "
             "cycles.  This checkbox does not have an effect upon whether "
@@ -134,7 +111,16 @@ class Mailbox(models.Model):
         default=True,
     )
 
-    objects = models.Manager()
+    last_polling = models.DateTimeField(
+        _("Last polling"),
+        help_text=(_("The time of last successful polling for messages."
+                     "It is blank for new mailboxes and is not set for "
+                     "mailboxes that only receive messages via a pipe.")),
+        blank=True,
+        null=True
+    )
+
+    objects = MailboxManager()
     active_mailboxes = ActiveMailboxManager()
 
     @property
@@ -151,22 +137,27 @@ class Mailbox(models.Model):
 
     @property
     def port(self):
+        """Returns the port to use for fetching messages."""
         return self._protocol_info.port
 
     @property
     def username(self):
+        """Returns the username to use for fetching messages."""
         return unquote(self._protocol_info.username)
 
     @property
     def password(self):
+        """Returns the password to use for fetching messages."""
         return unquote(self._protocol_info.password)
 
     @property
     def location(self):
+        """Returns the location (domain and path) of messages."""
         return self._domain if self._domain else '' + self._protocol_info.path
 
     @property
     def type(self):
+        """Returns the 'transport' name for this mailbox."""
         scheme = self._protocol_info.scheme.lower()
         if '+' in scheme:
             return scheme.split('+')[0]
@@ -174,16 +165,37 @@ class Mailbox(models.Model):
 
     @property
     def use_ssl(self):
+        """Returns whether or not this mailbox's connection uses SSL."""
         return '+ssl' in self._protocol_info.scheme.lower()
 
+    @property
+    def use_tls(self):
+        """Returns whether or not this mailbox's connection uses STARTTLS."""
+        return '+tls' in self._protocol_info.scheme.lower()
+
     @property
     def archive(self):
+        """Returns (if specified) the folder to archive messages to."""
         archive_folder = self._query_string.get('archive', None)
         if not archive_folder:
             return None
         return archive_folder[0]
 
+    @property
+    def folder(self):
+        """Returns (if specified) the folder to fetch mail from."""
+        folder = self._query_string.get('folder', None)
+        if not folder:
+            return None
+        return folder[0]
+
     def get_connection(self):
+        """Returns the transport instance for this mailbox.
+
+        These will always be instances of
+        `django_mailbox.transports.base.EmailTransport`.
+
+        """
         if not self.uri:
             return None
         elif self.type == 'imap':
@@ -191,7 +203,9 @@ class Mailbox(models.Model):
                 self.location,
                 port=self.port if self.port else None,
                 ssl=self.use_ssl,
-                archive=self.archive
+                tls=self.use_tls,
+                archive=self.archive,
+                folder=self.folder
             )
             conn.connect(self.username, self.password)
         elif self.type == 'gmail':
@@ -222,24 +236,29 @@ class Mailbox(models.Model):
         return conn
 
     def process_incoming_message(self, message):
+        """Process a message incoming to this mailbox."""
         msg = self._process_message(message)
+        if msg is None:
+            return None
         msg.outgoing = False
         msg.save()
 
-        try:
-            message_received.send(sender=self, message=msg)
-        except:
-            pass
+        message_received.send(sender=self, message=msg)
 
         return msg
 
     def record_outgoing_message(self, message):
+        """Record an outgoing message associated with this mailbox."""
         msg = self._process_message(message)
+        if msg is None:
+            return None
         msg.outgoing = True
         msg.save()
         return msg
 
     def _get_dehydrated_message(self, msg, record):
+        settings = utils.get_settings()
+
         new = EmailMessage()
         if msg.is_multipart():
             for header, value in msg.items():
@@ -249,22 +268,30 @@ class Mailbox(models.Model):
                     self._get_dehydrated_message(part, record)
                 )
         elif (
-            STRIP_UNALLOWED_MIMETYPES
-            and not msg.get_content_type() in ALLOWED_MIMETYPES
+            settings['strip_unallowed_mimetypes']
+            and not msg.get_content_type() in settings['allowed_mimetypes']
         ):
             for header, value in msg.items():
                 new[header] = value
             # Delete header, otherwise when attempting to  deserialize the
             # payload, it will be expecting a body for this.
             del new['Content-Transfer-Encoding']
-            new[ALTERED_MESSAGE_HEADER] = (
+            new[settings['altered_message_header']] = (
                 'Stripped; Content type %s not allowed' % (
                     msg.get_content_type()
                 )
             )
             new.set_payload('')
-        elif msg.get_content_type() not in TEXT_STORED_MIMETYPES:
-            filename = msg.get_filename()
+        elif (
+            (
+                msg.get_content_type() not in settings['text_stored_mimetypes']
+            ) or
+            ('attachment' in msg.get('Content-Disposition', ''))
+        ):
+            filename = None
+            raw_filename = msg.get_filename()
+            if raw_filename:
+                filename = utils.convert_header_to_unicode(raw_filename)
             if not filename:
                 extension = mimetypes.guess_extension(msg.get_content_type())
             else:
@@ -277,7 +304,7 @@ class Mailbox(models.Model):
             attachment.document.save(
                 uuid.uuid4().hex + extension,
                 ContentFile(
-                    six.BytesIO(
+                    BytesIO(
                         msg.get_payload(decode=True)
                     ).getvalue()
                 )
@@ -288,7 +315,9 @@ class Mailbox(models.Model):
             attachment.save()
 
             placeholder = EmailMessage()
-            placeholder[ATTACHMENT_INTERPOLATION_HEADER] = str(attachment.pk)
+            placeholder[
+                settings['attachment_interpolation_header']
+            ] = str(attachment.pk)
             new = placeholder
         else:
             content_charset = msg.get_content_charset()
@@ -312,12 +341,12 @@ class Mailbox(models.Model):
                 )
             except ValueError:
                 logger.warning(
-                    "Decoding error encountered; interpreting as ASCII!",
+                    "Decoding error encountered; interpreting %s as ASCII!",
                     content_charset
                 )
                 msg.set_payload(
                     msg.get_payload(decode=True).decode(
-                        content_charset,
+                        'ascii',
                         'ignore'
                     )
                 )
@@ -326,64 +355,106 @@ class Mailbox(models.Model):
 
     def _process_message(self, message):
         msg = Message()
+        msg._email_object = message
+        settings = utils.get_settings()
+
+        if settings['store_original_message']:
+            self._process_save_original_message(message, msg)
         msg.mailbox = self
         if 'subject' in message:
-            msg.subject = convert_header_to_unicode(message['subject'])[0:255]
+            msg.subject = (
+                utils.convert_header_to_unicode(message['subject'])[0:255]
+            )
         if 'message-id' in message:
-            msg.message_id = message['message-id'][0:255]
+            msg.message_id = message['message-id'][0:255].strip()
         if 'from' in message:
-            msg.from_header = convert_header_to_unicode(message['from'])
+            msg.from_header = utils.convert_header_to_unicode(message['from'])
         if 'to' in message:
-            msg.to_header = convert_header_to_unicode(message['to'])
+            msg.to_header = utils.convert_header_to_unicode(message['to'])
         elif 'Delivered-To' in message:
-            msg.to_header = convert_header_to_unicode(message['Delivered-To'])
+            msg.to_header = utils.convert_header_to_unicode(
+                message['Delivered-To']
+            )
         msg.save()
         message = self._get_dehydrated_message(message, msg)
-        msg.set_body(message.as_string())
+        try:
+            body = message.as_string()
+        except KeyError as exc:
+            # email.message.replace_header may raise 'KeyError' if the header
+            # 'content-transfer-encoding' is missing
+            logger.warning("Failed to parse message: %s", exc,)
+            return None
+        msg.set_body(body)
         if message['in-reply-to']:
             try:
                 msg.in_reply_to = Message.objects.filter(
-                    message_id=message['in-reply-to']
+                    message_id=message['in-reply-to'].strip()
                 )[0]
             except IndexError:
                 pass
         msg.save()
         return msg
 
-    def get_new_mail(self):
+    def _process_save_original_message(self, message, msg):
+        settings = utils.get_settings()
+        if settings['compress_original_message']:
+            with NamedTemporaryFile(suffix=".eml.gz") as fp_tmp:
+                with gzip.GzipFile(fileobj=fp_tmp, mode="w") as fp:
+                    fp.write(message.as_string().encode('utf-8'))
+                msg.eml.save(
+                    "{}.eml.gz".format(uuid.uuid4()),
+                    File(fp_tmp),
+                    save=False
+                )
+
+        else:
+            msg.eml.save(
+                '%s.eml' % uuid.uuid4(),
+                ContentFile(message.as_string()),
+                save=False
+            )
+
+    def get_new_mail(self, condition=None):
+        """Connect to this transport and fetch new messages."""
         new_mail = []
         connection = self.get_connection()
         if not connection:
-            return new_mail
-        for message in connection.get_message():
+            return
+        for message in connection.get_message(condition):
             msg = self.process_incoming_message(message)
-            new_mail.append(msg)
-        return new_mail
+            if not msg is None:
+                yield msg
+        self.last_polling = now()
+        if django.VERSION >= (1, 5):  # Django 1.5 introduces update_fields
+            self.save(update_fields=['last_polling'])
+        else:
+            self.save()
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     class Meta:
-        verbose_name_plural = "Mailboxes"
+        verbose_name = _('Mailbox')
+        verbose_name_plural = _('Mailboxes')
 
 
 class IncomingMessageManager(models.Manager):
     def get_queryset(self):
-        return super(IncomingMessageManager, self).get_queryset().filter(
+        return super().get_queryset().filter(
             outgoing=False,
         )
 
 
 class OutgoingMessageManager(models.Manager):
     def get_queryset(self):
-        return super(OutgoingMessageManager, self).get_queryset().filter(
+        return super().get_queryset().filter(
             outgoing=True,
         )
 
 
 class UnreadMessageManager(models.Manager):
     def get_queryset(self):
-        return super(UnreadMessageManager, self).get_queryset().filter(
+        return super().get_queryset().filter(
             read=None
         )
 
@@ -392,16 +463,17 @@ class Message(models.Model):
     mailbox = models.ForeignKey(
         Mailbox,
         related_name='messages',
-        verbose_name=_(u'Mailbox'),
+        verbose_name=_('Mailbox'),
+        on_delete=models.CASCADE
     )
 
     subject = models.CharField(
-        _(u'Subject'),
+        _('Subject'),
         max_length=255
     )
 
     message_id = models.CharField(
-        _(u'Message ID'),
+        _('Message ID'),
         max_length=255
     )
 
@@ -410,7 +482,8 @@ class Message(models.Model):
         related_name='replies',
         blank=True,
         null=True,
-        verbose_name=_(u'In reply to'),
+        verbose_name=_('In reply to'),
+        on_delete=models.CASCADE
     )
 
     from_header = models.CharField(
@@ -419,21 +492,21 @@ class Message(models.Model):
     )
 
     to_header = models.TextField(
-        _(u'To header'),
+        _('To header'),
     )
 
     outgoing = models.BooleanField(
-        _(u'Outgoing'),
+        _('Outgoing'),
         default=False,
         blank=True,
     )
 
     body = models.TextField(
-        _(u'Body'),
+        _('Body'),
     )
 
     encoded = models.BooleanField(
-        _(u'Encoded'),
+        _('Encoded'),
         default=False,
         help_text=_('True if the e-mail body is Base64 encoded'),
     )
@@ -444,12 +517,18 @@ class Message(models.Model):
     )
 
     read = models.DateTimeField(
-        _(u'Read'),
+        _('Read'),
         default=None,
         blank=True,
         null=True,
     )
 
+    eml = models.FileField(
+        _('Raw message contents'),
+        null=True,
+        upload_to="messages",
+        help_text=_('Original full content of message')
+    )
     objects = models.Manager()
     unread_messages = UnreadMessageManager()
     incoming_messages = IncomingMessageManager()
@@ -472,6 +551,16 @@ class Message(models.Model):
 
     @property
     def from_address(self):
+        """Returns the address (as a list) from which this message was received
+
+        .. note::
+
+           This was once (and probably should be) a string rather than a list,
+           but in a pull request received long, long ago it was changed;
+           presumably to make the interface identical to that of
+           `to_addresses`.
+
+        """
         if self.from_header:
             return [parseaddr(self.from_header)[1].lower()]
         else:
@@ -479,6 +568,7 @@ class Message(models.Model):
 
     @property
     def to_addresses(self):
+        """Returns a list of addresses to which this message was sent."""
         addresses = []
         for address in self.to_header.split(','):
             if address:
@@ -498,13 +588,14 @@ class Message(models.Model):
         pre-set it.
 
         """
-        if self.mailbox.from_email:
-            message.from_email = self.mailbox.from_email
-        else:
-            message.from_email = settings.DEFAULT_FROM_EMAIL
+        if not message.from_email:
+            if self.mailbox.from_email:
+                message.from_email = self.mailbox.from_email
+            else:
+                message.from_email = django_settings.DEFAULT_FROM_EMAIL
         message.extra_headers['Message-ID'] = make_msgid()
         message.extra_headers['Date'] = formatdate()
-        message.extra_headers['In-Reply-To'] = self.message_id
+        message.extra_headers['In-Reply-To'] = self.message_id.strip()
         message.send()
         return self.mailbox.record_outgoing_message(
             email.message_from_string(
@@ -517,7 +608,7 @@ class Message(models.Model):
         """
         Returns the message body matching content type 'text/plain'.
         """
-        return get_body_from_message(
+        return utils.get_body_from_message(
             self.get_email_object(), 'text', 'plain'
         ).replace('=\n', '').strip()
 
@@ -526,12 +617,14 @@ class Message(models.Model):
         """
         Returns the message body matching content type 'text/html'.
         """
-        return get_body_from_message(
+        return utils.get_body_from_message(
             self.get_email_object(), 'text', 'html'
         ).replace('\n', '').strip()
 
     def _rehydrate(self, msg):
         new = EmailMessage()
+        settings = utils.get_settings()
+
         if msg.is_multipart():
             for header, value in msg.items():
                 new[header] = value
@@ -539,10 +632,10 @@ class Message(models.Model):
                 new.attach(
                     self._rehydrate(part)
                 )
-        elif ATTACHMENT_INTERPOLATION_HEADER in msg.keys():
+        elif settings['attachment_interpolation_header'] in msg.keys():
             try:
                 attachment = MessageAttachment.objects.get(
-                    pk=msg[ATTACHMENT_INTERPOLATION_HEADER]
+                    pk=msg[settings['attachment_interpolation_header']]
                 )
                 for header, value in attachment.items():
                     new[header] = value
@@ -550,9 +643,9 @@ class Message(models.Model):
                 if encoding and encoding.lower() == 'quoted-printable':
                     # Cannot use `email.encoders.encode_quopri due to
                     # bug 14360: http://bugs.python.org/issue14360
-                    output = six.BytesIO()
+                    output = BytesIO()
                     encode_quopri(
-                        six.BytesIO(
+                        BytesIO(
                             attachment.document.read()
                         ),
                         output,
@@ -571,9 +664,9 @@ class Message(models.Model):
                     del new['Content-Transfer-Encoding']
                     encode_base64(new)
             except MessageAttachment.DoesNotExist:
-                new[ALTERED_MESSAGE_HEADER] = (
+                new[settings['altered_message_header']] = (
                     'Missing; Attachment %s not found' % (
-                        msg[ATTACHMENT_INTERPOLATION_HEADER]
+                        msg[settings['attachment_interpolation_header']]
                     )
                 )
                 new.set_payload('')
@@ -586,34 +679,74 @@ class Message(models.Model):
         return new
 
     def get_body(self):
+        """Returns the `body` field of this record.
+
+        This will automatically base64-decode the message contents
+        if they are encoded as such.
+
+        """
         if self.encoded:
             return base64.b64decode(self.body.encode('ascii'))
         return self.body.encode('utf-8')
 
     def set_body(self, body):
-        if six.PY3:
-            body = body.encode('utf-8')
+        """Set the `body` field of this record.
+
+        This will automatically base64-encode the message contents to
+        circumvent a limitation in earlier versions of Django in which
+        no fields existed for storing arbitrary bytes.
+
+        """
         self.encoded = True
-        self.body = base64.b64encode(body).decode('ascii')
+        self.body = base64.b64encode(body.encode('utf-8')).decode('ascii')
 
     def get_email_object(self):
-        """ Returns an `email.message.Message` instance for this message."""
-        body = self.get_body()
-        if six.PY3:
+        """Returns an `email.message.EmailMessage` instance representing the
+        contents of this message and all attachments.
+
+        See [email.message.EmailMessage]_ for more information as to what methods
+        and properties are available on `email.message.EmailMessage` instances.
+
+        .. note::
+
+           Depending upon the storage methods in use (specifically --
+           whether ``DJANGO_MAILBOX_STORE_ORIGINAL_MESSAGE`` is set
+           to ``True``, this may either create a "rehydrated" message
+           using stored attachments, or read the message contents stored
+           on-disk.
+
+        .. [email.message.EmailMessage] Python's `email.message.EmailMessage` docs
+           (https://docs.python.org/3/library/email.message.html)
+
+        """
+        if not hasattr(self, '_email_object'):  # Cache fill
+            if self.eml:
+                if self.eml.name.endswith('.gz'):
+                    body = gzip.GzipFile(fileobj=self.eml).read()
+                else:
+                    self.eml.open()
+                    body = self.eml.file.read()
+                    self.eml.close()
+            else:
+                body = self.get_body()
             flat = email.message_from_bytes(body)
-        else:
-            flat = email.message_from_string(body)
-        return self._rehydrate(flat)
+            self._email_object = self._rehydrate(flat)
+        return self._email_object
 
     def delete(self, *args, **kwargs):
+        """Delete this message and all stored attachments."""
         for attachment in self.attachments.all():
             # This attachment is attached only to this message.
             attachment.delete()
-        return super(Message, self).delete(*args, **kwargs)
+        return super().delete(*args, **kwargs)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.subject
 
+    class Meta:
+        verbose_name = _('E-mail message')
+        verbose_name_plural = _('E-mail messages')
+
 
 class MessageAttachment(models.Model):
     message = models.ForeignKey(
@@ -622,29 +755,29 @@ class MessageAttachment(models.Model):
         null=True,
         blank=True,
         verbose_name=_('Message'),
+        on_delete=models.CASCADE
     )
 
     headers = models.TextField(
-        _(u'Headers'),
+        _('Headers'),
         null=True,
         blank=True,
     )
 
     document = models.FileField(
-        _(u'Document'),
-        upload_to='mailbox_attachments/%Y/%m/%d/'
+        _('Document'),
+        upload_to=utils.get_attachment_save_path,
     )
 
     def delete(self, *args, **kwargs):
+        """Deletes the attachment."""
         self.document.delete()
-        return super(MessageAttachment, self).delete(*args, **kwargs)
+        return super().delete(*args, **kwargs)
 
     def _get_rehydrated_headers(self):
         headers = self.headers
         if headers is None:
             return EmailMessage()
-        if sys.version_info < (3, 0):
-            headers = headers.encode('utf-8')
         return email.message_from_string(headers)
 
     def _set_dehydrated_headers(self, email_object):
@@ -661,11 +794,13 @@ class MessageAttachment(models.Model):
         self._set_dehydrated_headers(rehydrated)
 
     def get_filename(self):
+        """Returns the original filename of this attachment."""
         file_name = self._get_rehydrated_headers().get_filename()
-        if isinstance(file_name, six.text_type):
-            return file_name
-        elif file_name:
-            return convert_header_to_unicode(file_name)
+        if isinstance(file_name, str):
+            result = utils.convert_header_to_unicode(file_name)
+            if result is None:
+                return file_name
+            return result
         else:
             return None
 
@@ -678,5 +813,9 @@ class MessageAttachment(models.Model):
             raise KeyError('Header %s does not exist' % name)
         return value
 
-    def __unicode__(self):
+    def __str__(self):
         return self.document.url
+
+    class Meta:
+        verbose_name = _('Message attachment')
+        verbose_name_plural = _('Message attachments')
